'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _colr = require('colr');

var _colr2 = _interopRequireDefault(_colr);

var _addEventListener = require('rc-util/lib/Dom/addEventListener');

var _addEventListener2 = _interopRequireDefault(_addEventListener);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _defaults(obj, defaults) { var keys = Object.getOwnPropertyNames(defaults); for (var i = 0; i < keys.length; i++) { var key = keys[i]; var value = Object.getOwnPropertyDescriptor(defaults, key); if (value && value.configurable && obj[key] === undefined) { Object.defineProperty(obj, key, value); } } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : _defaults(subClass, superClass); }

var colr = new _colr2["default"]();

function rgbaColor(r, g, b, a) {
  return 'rgba(' + [r, g, b, a / 100].join(',') + ')';
}

var Alpha = function (_React$Component) {
  _inherits(Alpha, _React$Component);

  function Alpha(props) {
    _classCallCheck(this, Alpha);

    var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

    var events = ['onMouseDown', 'onDrag', 'onDragEnd', 'pointMoveTo', 'getBackground'];
    events.forEach(function (e) {
      _this[e] = _this[e].bind(_this);
    });
    return _this;
  }

  Alpha.prototype.componentWillUnmount = function componentWillUnmount() {
    this.removeListeners();
  };

  Alpha.prototype.onMouseDown = function onMouseDown(e) {
    var x = e.clientX;
    var y = e.clientY;

    this.pointMoveTo({
      x: x, y: y
    });

    this.dragListener = (0, _addEventListener2["default"])(window, 'mousemove', this.onDrag);
    this.dragUpListener = (0, _addEventListener2["default"])(window, 'mouseup', this.onDragEnd);
  };

  Alpha.prototype.onDrag = function onDrag(e) {
    var x = e.clientX;
    var y = e.clientY;
    this.pointMoveTo({
      x: x, y: y
    });
  };

  Alpha.prototype.onDragEnd = function onDragEnd(e) {
    var x = e.clientX;
    var y = e.clientY;
    this.pointMoveTo({
      x: x, y: y
    });
    this.removeListeners();
  };

  Alpha.prototype.getBackground = function getBackground() {
    var _colr$fromHsvObject$t = colr.fromHsvObject(this.props.hsv).toRgbObject(),
        r = _colr$fromHsvObject$t.r,
        g = _colr$fromHsvObject$t.g,
        b = _colr$fromHsvObject$t.b;

    var opacityGradient = 'linear-gradient(to right, ' + rgbaColor(r, g, b, 0) + ' , ' + rgbaColor(r, g, b, 100) + ')'; // eslint-disable-line max-len
    return opacityGradient;
  };

  Alpha.prototype.getPrefixCls = function getPrefixCls() {
    return this.props.rootPrefixCls + '-alpha';
  };

  Alpha.prototype.pointMoveTo = function pointMoveTo(coords) {
    var rect = _reactDom2["default"].findDOMNode(this).getBoundingClientRect();
    var width = rect.width;
    var left = coords.x - rect.left;

    left = Math.max(0, left);
    left = Math.min(left, width);

    var alpha = Math.floor(left / width * 100);

    this.props.onChange(alpha);
  };

  Alpha.prototype.removeListeners = function removeListeners() {
    if (this.dragListener) {
      this.dragListener.remove();
      this.dragListener = null;
    }
    if (this.dragUpListener) {
      this.dragUpListener.remove();
      this.dragUpListener = null;
    }
  };

  Alpha.prototype.render = function render() {
    var prefixCls = this.getPrefixCls();
    return _react2["default"].createElement(
      'div',
      { className: prefixCls },
      _react2["default"].createElement('div', {
        ref: 'bg',
        className: prefixCls + '-bg',
        style: { background: this.getBackground() }
      }),
      _react2["default"].createElement('span', { style: { left: this.props.alpha + '%' } }),
      _react2["default"].createElement('div', {
        className: prefixCls + '-handler',
        onMouseDown: this.onMouseDown
      })
    );
  };

  return Alpha;
}(_react2["default"].Component);

exports["default"] = Alpha;


Alpha.propTypes = {
  hsv: _propTypes2["default"].object,
  onChange: _propTypes2["default"].func,
  rootPrefixCls: _propTypes2["default"].string,
  alpha: _propTypes2["default"].number
};
module.exports = exports['default'];