'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _colr = require('colr');

var _colr2 = _interopRequireDefault(_colr);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _reactDom2 = _interopRequireDefault(_reactDom);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _addEventListener = require('rc-util/lib/Dom/addEventListener');

var _addEventListener2 = _interopRequireDefault(_addEventListener);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _defaults(obj, defaults) { var keys = Object.getOwnPropertyNames(defaults); for (var i = 0; i < keys.length; i++) { var key = keys[i]; var value = Object.getOwnPropertyDescriptor(defaults, key); if (value && value.configurable && obj[key] === undefined) { Object.defineProperty(obj, key, value); } } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : _defaults(subClass, superClass); }

var colr = new _colr2["default"]();

var WIDTH = 200;
var HEIGHT = 150;

var Board = function (_React$Component) {
  _inherits(Board, _React$Component);

  function Board(props) {
    _classCallCheck(this, Board);

    var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

    var events = ['onBoardMouseDown', 'onBoardDrag', 'onBoardDragEnd', 'onBoardTouchStart', 'onBoardTouchMove', 'onBoardTouchEnd'];
    events.forEach(function (m) {
      _this[m] = _this[m].bind(_this);
    });
    return _this;
  }

  Board.prototype.componentWillUnmount = function componentWillUnmount() {
    this.removeListeners();
    this.removeTouchListeners();
  };

  Board.prototype.onBoardMouseDown = function onBoardMouseDown(e) {
    var x = e.clientX;
    var y = e.clientY;
    this.pointMoveTo({
      x: x,
      y: y
    });
    this.dragListener = (0, _addEventListener2["default"])(window, 'mousemove', this.onBoardDrag);
    this.dragUpListener = (0, _addEventListener2["default"])(window, 'mouseup', this.onBoardDragEnd);
  };

  Board.prototype.onBoardTouchStart = function onBoardTouchStart(e) {
    if (e.touches.length !== 1) {
      return;
    }

    var x = e.targetTouches[0].clientX;
    var y = e.targetTouches[0].clientY;
    this.pointMoveTo({
      x: x,
      y: y
    });
    this.touchMoveListener = (0, _addEventListener2["default"])(window, 'touchmove', this.onBoardTouchMove);
    this.touchEndListener = (0, _addEventListener2["default"])(window, 'touchend', this.onBoardTouchEnd);
  };

  Board.prototype.onBoardTouchMove = function onBoardTouchMove(e) {
    if (e.preventDefault) {
      e.preventDefault();
    }

    var x = e.targetTouches[0].clientX;
    var y = e.targetTouches[0].clientY;
    this.pointMoveTo({
      x: x,
      y: y
    });
  };

  Board.prototype.onBoardTouchEnd = function onBoardTouchEnd() {
    this.removeTouchListeners();
  };

  Board.prototype.onBoardDrag = function onBoardDrag(e) {
    var x = e.clientX;
    var y = e.clientY;
    this.pointMoveTo({
      x: x,
      y: y
    });
  };

  Board.prototype.onBoardDragEnd = function onBoardDragEnd(e) {
    var x = e.clientX;
    var y = e.clientY;
    this.pointMoveTo({
      x: x,
      y: y
    });
    this.removeListeners();
  };

  Board.prototype.getPrefixCls = function getPrefixCls() {
    return this.props.rootPrefixCls + '-board';
  };

  Board.prototype.removeTouchListeners = function removeTouchListeners() {
    if (this.touchMoveListener) {
      this.touchMoveListener.remove();
      this.touchMoveListener = null;
    }
    if (this.touchEndListener) {
      this.touchEndListener.remove();
      this.touchEndListener = null;
    }
  };

  Board.prototype.removeListeners = function removeListeners() {
    if (this.dragListener) {
      this.dragListener.remove();
      this.dragListener = null;
    }
    if (this.dragUpListener) {
      this.dragUpListener.remove();
      this.dragUpListener = null;
    }
  };

  /**
   * 移动光标位置到
   * @param  {object} pos X Y 全局坐标点
   * @return {undefined}
   */


  Board.prototype.pointMoveTo = function pointMoveTo(pos) {
    var rect = _reactDom2["default"].findDOMNode(this).getBoundingClientRect();
    var left = pos.x - rect.left;
    var top = pos.y - rect.top;

    left = Math.max(0, left);
    left = Math.min(left, WIDTH);
    top = Math.max(0, top);
    top = Math.min(top, HEIGHT);

    var hsv = {
      h: this.props.hsv.h,
      s: parseInt(left / WIDTH * 100, 10),
      v: parseInt((1 - top / HEIGHT) * 100, 10)
    };
    this.props.onChange(hsv);
  };

  Board.prototype.render = function render() {
    var prefixCls = this.getPrefixCls();
    var hsv = this.props.hsv;
    var hueHsv = [hsv.h, 100, 100];
    var hueColor = colr.fromHsvArray(hueHsv).toHex();
    var x = hsv.s / 100 * WIDTH - 4;
    var y = (1 - hsv.v / 100) * HEIGHT - 4;
    return _react2["default"].createElement(
      'div',
      { className: prefixCls },
      _react2["default"].createElement(
        'div',
        { className: prefixCls + '-hsv', style: { backgroundColor: hueColor } },
        _react2["default"].createElement('div', { className: prefixCls + '-value' }),
        _react2["default"].createElement('div', { className: prefixCls + '-saturation' })
      ),
      _react2["default"].createElement('span', { style: { left: x, top: y } }),
      _react2["default"].createElement('div', {
        className: prefixCls + '-handler',
        onMouseDown: this.onBoardMouseDown,
        onTouchStart: this.onBoardTouchStart
      })
    );
  };

  return Board;
}(_react2["default"].Component);

exports["default"] = Board;


Board.propTypes = {
  hsv: _propTypes2["default"].object,
  onChange: _propTypes2["default"].func,
  rootPrefixCls: _propTypes2["default"].string
};
module.exports = exports['default'];