'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _colr = require('colr');

var _colr2 = _interopRequireDefault(_colr);

var _Board = require('./Board');

var _Board2 = _interopRequireDefault(_Board);

var _Preview = require('./Preview');

var _Preview2 = _interopRequireDefault(_Preview);

var _Ribbon = require('./Ribbon');

var _Ribbon2 = _interopRequireDefault(_Ribbon);

var _Alpha = require('./Alpha');

var _Alpha2 = _interopRequireDefault(_Alpha);

var _Params = require('./Params');

var _Params2 = _interopRequireDefault(_Params);

var _validationColor = require('./utils/validationColor');

var _validationColor2 = _interopRequireDefault(_validationColor);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _defaults(obj, defaults) { var keys = Object.getOwnPropertyNames(defaults); for (var i = 0; i < keys.length; i++) { var key = keys[i]; var value = Object.getOwnPropertyDescriptor(defaults, key); if (value && value.configurable && obj[key] === undefined) { Object.defineProperty(obj, key, value); } } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : _defaults(subClass, superClass); }

function noop() {}

var colr = new _colr2["default"]();

var Panel = function (_React$Component) {
  _inherits(Panel, _React$Component);

  function Panel(props) {
    _classCallCheck(this, Panel);

    var _this = _possibleConstructorReturn(this, _React$Component.call(this, props));

    var color = props.color || props.defaultColor;
    var hsv = colr.fromHex(color).toHsvObject();

    var alpha = typeof props.alpha === 'undefined' ? props.defaultAlpha : Math.min(props.alpha, props.defaultAlpha);

    _this.state = {
      paramsHsv: hsv,
      hsv: hsv,
      alpha: alpha
    };

    var events = ['onChange', 'onAlphaChange', 'onFocus', 'onBlur', 'onSystemColorPickerOpen'];
    // bind methods
    events.forEach(function (m) {
      _this[m] = _this[m].bind(_this);
    });
    return _this;
  }

  Panel.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
    if (nextProps.color) {
      var hsv = colr.fromHex(nextProps.color).toHsvObject();
      this.setState({
        hsv: hsv,
        paramsHsv: hsv
      });
    }
    if (nextProps.alpha !== undefined) {
      this.setState({
        alpha: nextProps.alpha
      });
    }
  };

  Panel.prototype.onChange = function onChange(hsvObject) {
    var syncParams = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

    var hsv = hsvObject;
    var state = {
      hsv: hsv
    };
    if (syncParams) {
      state.paramsHsv = hsv;
    }
    this.setState(state);

    var ret = {
      color: this.getHexColor(hsv),
      hsv: hsv,
      alpha: this.state.alpha
    };
    this.props.onChange(ret);
  };

  Panel.prototype.onSystemColorPickerOpen = function onSystemColorPickerOpen(e) {
    // only work with broswer which support color input
    if (e.target.type === 'color') {
      this.systemColorPickerOpen = true;
    }
  };

  Panel.prototype.onAlphaChange = function onAlphaChange(alpha) {
    if (this.props.alpha === undefined) {
      this.setState({
        alpha: alpha
      });
    }
    this.props.onChange({
      color: this.getHexColor(),
      hsv: this.state.hsv,
      alpha: alpha
    });
  };

  Panel.prototype.onFocus = function onFocus() {
    if (this._blurTimer) {
      clearTimeout(this._blurTimer);
      this._blurTimer = null;
    } else {
      this.props.onFocus();
    }
  };

  Panel.prototype.onBlur = function onBlur() {
    var _this2 = this;

    if (this._blurTimer) {
      clearTimeout(this._blurTimer);
    }
    this._blurTimer = setTimeout(function () {
      // if is system color picker open, then stop run blur
      if (_this2.systemColorPickerOpen) {
        _this2.systemColorPickerOpen = false;
        return;
      }

      _this2.props.onBlur();
    }, 100);
  };

  Panel.prototype.getHexColor = function getHexColor(hsv) {
    return colr.fromHsvObject(hsv || this.state.hsv).toHex();
  };

  Panel.prototype.render = function render() {
    var prefixCls = this.props.prefixCls;
    var hsv = this.state.hsv;
    var alpha = this.state.alpha;
    return _react2["default"].createElement(
      'div',
      {
        className: [prefixCls, this.props.className].join(' '),
        style: this.props.style,
        onFocus: this.onFocus,
        onBlur: this.onBlur,
        tabIndex: '0'
      },
      _react2["default"].createElement(
        'div',
        { className: prefixCls + '-inner' },
        _react2["default"].createElement(_Board2["default"], {
          rootPrefixCls: prefixCls,
          hsv: hsv,
          onChange: this.onChange
        }),
        _react2["default"].createElement(
          'div',
          { className: prefixCls + '-wrap' },
          _react2["default"].createElement(
            'div',
            { className: prefixCls + '-wrap-ribbon' },
            _react2["default"].createElement(_Ribbon2["default"], {
              rootPrefixCls: prefixCls,
              hsv: hsv,
              onChange: this.onChange
            })
          ),
          _react2["default"].createElement(
            'div',
            { className: prefixCls + '-wrap-alpha' },
            _react2["default"].createElement(_Alpha2["default"], {
              rootPrefixCls: prefixCls,
              alpha: alpha,
              hsv: hsv,
              onChange: this.onAlphaChange
            })
          ),
          _react2["default"].createElement(
            'div',
            { className: prefixCls + '-wrap-preview' },
            _react2["default"].createElement(_Preview2["default"], {
              rootPrefixCls: prefixCls,
              alpha: alpha,
              onChange: this.onChange,
              onInputClick: this.onSystemColorPickerOpen,
              hsv: hsv
            })
          )
        ),
        _react2["default"].createElement(
          'div',
          { className: prefixCls + '-wrap', style: { height: 40, marginTop: 6 } },
          _react2["default"].createElement(_Params2["default"], {
            rootPrefixCls: prefixCls,
            hsv: this.state.paramsHsv,
            alpha: alpha,
            onAlphaChange: this.onAlphaChange,
            onChange: this.onChange,
            mode: this.props.mode
          })
        )
      )
    );
  };

  return Panel;
}(_react2["default"].Component);

exports["default"] = Panel;


Panel.propTypes = {
  defaultAlpha: _propTypes2["default"].number,
  defaultColor: _validationColor2["default"],
  // can custom
  prefixCls: _propTypes2["default"].string,
  color: _validationColor2["default"],
  alpha: _propTypes2["default"].number,
  style: _propTypes2["default"].object,
  onChange: _propTypes2["default"].func,
  onFocus: _propTypes2["default"].func,
  onBlur: _propTypes2["default"].func,
  mode: _propTypes2["default"].oneOf(['RGB', 'HSL', 'HSB']),
  className: _propTypes2["default"].string
};

Panel.defaultProps = {
  prefixCls: 'rc-color-picker-panel',
  defaultColor: '#ff0000',
  defaultAlpha: 100,
  style: {},
  onChange: noop,
  onFocus: noop,
  onBlur: noop,
  mode: 'RGB',
  className: ''
};
module.exports = exports['default'];